package seamCarving;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;

import javax.imageio.ImageIO;
import javax.swing.JPanel;

public class ImageDisplay extends JPanel {

    private ImageMatrix imageMatrix;
    private BufferedImage origImage;

    private int width;
    private int height;

    /**
     * Read an image file into memory.
     * @param filename
     */
    public ImageDisplay(String filename) {
        super();

        // Read in the image
        try {
            this.origImage = ImageIO.read(new File(filename));
            this.imageMatrix = new ImageMatrix(this.origImage);

            // Set up the size
            this.width = this.origImage.getWidth();
            this.height = this.origImage.getHeight();

            this.setPreferredSize(new Dimension(this.width + 50, this.height + 50));
        } catch (IOException e) {
            System.err.println(e);
        }
    }

    public void calculateAndShowBestSeam() {
        // Find the best seam
        this.imageMatrix.findBestSeam();

        // Display the seam 
        this.imageMatrix.colorSeam();
    }
    
    public void removeBestSeam() {
        this.imageMatrix.removeSeam();
    }

    @Override
    public void paintComponent(Graphics g) {
        super.paintComponent(g);

        BufferedImage image = this.imageMatrix.getImage();
        g.drawImage(image, 25, Math.max(0, (this.getHeight() - this.height) / 2), null);
    }

    public void reset() {
        this.imageMatrix = new ImageMatrix(this.origImage);
    }
    
    public int getOriginalWidth() {
        return this.origImage.getWidth();
    }
}
