package seamCarving;

import java.awt.BorderLayout;
import java.awt.EventQueue;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.Timer;
import java.util.TimerTask;

import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

public class Main {

    // TODO: Change this to change which image you are using
    public static final String imageName = "prob1/images/eiffeltower.png";
//    public static final String imageName = "prob1/images/flower.png";
//    public static final String imageName = "prob1/images/oldwell_small.png";
//    public static final String imageName = "prob1/images/oldwell_large.png";
//    public static final String imageName = "prob1/images/pisa.png";
//    public static final String imageName = "prob1/images/ramses.png";
//    public static final String imageName = "prob1/images/shanghai.png";

    // TODO: Change this to change how much of the image will be removed
    public static final double removeFraction = 0.5;

    // TODO: Change this to change how fast the steps are taken
    // (note: if this gets too small, you will get NullPointerExceptions
    // if the interrupts happen faster than the calls to remove the seam)
    public static final int delay = 100;

    private static JFrame frame;

    private static ImageDisplay display;

    private static int numSteps;
    private static int timeStep;

    private static final Timer timer = new Timer();
    private static TimerTask currentTask = null;

    private static boolean paused = true;

    private static JLabel stepLabel;

    private static JButton startButton;
    private static JButton resetButton;

    public static void main(String[] args) {
        EventQueue.invokeLater(new Runnable() {
            public void run() {
                startProgram();
            }
        });
    }

    private static void startProgram() {
        frame = new JFrame("Seam Carving");

        display = new ImageDisplay(imageName);

        setLayout();

        reset();

        scheduleTimerTask();
    }

    /**
     * Step forward by finding a seam and removing it.
     */
    private static void step() {
        if (timeStep % 2 == 0) {
            // On even steps, calculate the best seam and display it
            display.calculateAndShowBestSeam();
        } else {
            // On odd steps, remove the current best seam
            display.removeBestSeam();
        }

        frame.repaint();
    }

    private static void scheduleTimerTask() {
        currentTask = new TimerTask() {
            public void run() {
                if (paused) { return; }

                // On even time steps, update the step label
                if (timeStep % 2 == 0) {
                    stepLabel.setText(" Steps taken: " + String.valueOf(timeStep / 2));
                }

                if (timeStep < numSteps) {
                    step();
                    timeStep++;
                } else {
                    System.out.println("Done.");

                    // Cancel the task
                    this.cancel();
                }
            }
        };

        timer.schedule(currentTask,  delay, delay);
    }
    
    private static void reset() {
        numSteps = (int)(display.getOriginalWidth() * removeFraction);
        timeStep = 0;

        stepLabel.setText(" Steps taken: " + String.valueOf(timeStep));

        pause();

        display.reset();

        frame.repaint();
    }

    private static void start() {
        paused = false;
        startButton.setText("Pause");

        resetButton.setEnabled(false);
    }

    private static void pause() {
        paused = true;
        startButton.setText("Start");

        resetButton.setEnabled(true);
    }

    private static void setLayout() {
        frame.getContentPane().removeAll();

        frame.setLayout(new BorderLayout());
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        frame.add(display, BorderLayout.CENTER);

        addButtons();

        frame.pack();
        frame.validate();
        frame.setVisible(true);
    }

    private static void addButtons() {
        startButton = new JButton("Start");
        startButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (paused) {
                    start();
                } else {
                    pause();
                }
            }
        });

        resetButton = new JButton("Reset");
        resetButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                System.out.println("resetting");
                reset();
            }
        });

        JPanel buttonPanel = new JPanel();
        buttonPanel.add(startButton, BorderLayout.WEST);
        buttonPanel.add(resetButton, BorderLayout.CENTER);

        frame.add(buttonPanel, BorderLayout.NORTH);

        stepLabel = new JLabel();
        frame.add(stepLabel,  BorderLayout.SOUTH);
    }
}
